import dayjs from "dayjs"
import {LocationQueryValue} from "vue-router"
import {QueryConverter} from "@/components/UiScrollListModel"
import {uiComponentConfig} from "@/config/uiComponentConfig"

export type DatePeriod = {
  fromDate?: Date
  toDate?: Date
}

const serializeDate = (date?: Date, format = 'YYYYMMDD'): string => {
  return date ? dayjs(date).format(format) : ''
}

const deserializeDate = (str: string, endOfDate = false, format = 'YYYYMMDD'): Date | undefined => {
  if (str) {
    const date = dayjs(str, format)
    return endOfDate ? date.endOf('date').toDate() : date.toDate()
  } else {
    return undefined
  }
}

/**
 * DatePeriod 객체에 toString, toQueryValue, fromQueryValue 함수를 설정한다.
 * @param period
 */
export const enhanceDatePeriodObject = (period: DatePeriod): void => {

  period.toString = (): string => `${serializeDate(period.fromDate) || ''}_${serializeDate(period.toDate)}`

}

export const datePeriodConverter: QueryConverter<DatePeriod> = {

  toQueryString(value?: DatePeriod): string {
    const period = value as DatePeriod
    return `${serializeDate(period?.fromDate)}_${serializeDate(period?.toDate)}`
  },

  fromQueryString(query: LocationQueryValue | LocationQueryValue[]): DatePeriod | undefined {
    // 첫 번째 항목만 사용
    const str = String(Array.isArray(query) ? query[0] : query)
    if (str.includes('_')) {
      const dates = str.split('_')
      return {
        fromDate: deserializeDate(dates[0]),
        toDate: deserializeDate(dates[1], true)
      }
    }
    return undefined
  }
}

/**
 * months 로부터 DatePeriod 객체를 생성한다.
 * 예를 들어 months=3 이면 3개월 전부터 현재까지의 DatePeriod 가 생성된다.
 */
export const createDatePeriodByMonths = (months: number): DatePeriod => {
  const fromDate = dayjs().subtract(months, 'month').startOf('date').toDate()
  const toDate = dayjs().endOf('date').toDate()
  return {fromDate, toDate}
}

/**
 * UiDatePeriod 컴포넌트의 초기 값을 가져온다.
 */
export const getPeriodSearchInitialValue = (monthListItem?: MonthListItem): DatePeriod | undefined => {
  const month = monthListItem || uiComponentConfig.periodSearchMonthList?.[0]
  return (month && month.months) ? createDatePeriodByMonths(month.months) : undefined
}

export type MonthListItem = {
  caption: string
  months?: number
}

/**
 * 두 DatePeriod 객체가 동일한지 확인한다.
 * @param a
 * @param b
 */
export const isDatePeriodEqual = (a?: DatePeriod, b?: DatePeriod): boolean => {
  if (a === undefined && b === undefined) return true
  else if (a === undefined || b === undefined) return false
  else return JSON.stringify(a, Object.keys(a).sort()) === JSON.stringify(b, Object.keys(b).sort())
}