import dayjs, {Dayjs} from 'dayjs'
import {uiComponentConfig} from "@/config/uiComponentConfig"

const DATE_PATTERN = new Map<string, string>([
  ['DATE', 'YYYY-MM-DD'],
  ['MINUTE', 'YYYY-MM-DD HH:mm'],
  ['SECOND', 'YYYY-MM-DD HH:mm:ss']
])

/**
 *  소숫점 n+1 자리에서 반올림한다.
 */
const round = (num: number, decimalPlace = 2) => Math.round(num * Math.pow(10, decimalPlace)) / Math.pow(10, decimalPlace)

export interface FormatUtil {

  escapeNewLine(str?: string): string

  escapeHtml(str?: string): string

  splitNewLine(str?: string): Array<string>

  formatDate(date?: string | Date | Dayjs | null, pattern?: string): string

  formatPrice(value?: number | string | null, withUnit?: boolean): string | null

  formatPoint(value?: number | string | null, withUnit?: boolean): string | null

  formatNumber(value?: number | string | null): string | null

  formatPercent(value?: number | string | null, withUnit?: boolean, decimalPlace?: number): string | null

  formatCreditCardNo(value: string, separator?: string): string | undefined

  toPlainText(textIncludedHTMLElement?: string): string
}

const formatUtil: FormatUtil = {

  /**
   * 개행 문자('\r', '\n')를 '<br/>' 태그로 변경한다.
   * @param str 변경할 String
   * @return 개행문자가 '<br/>'로 변경된 String
   */
  escapeNewLine(str?: string): string {
    return str ? str.replace(/\r\n|\r|\n/g, '<br />') : ''
  },

  /**
   * `str` 안의 `&<>"'` 문자를 escape 처리한다.
   * @param str 변경할 String
   */
  escapeHtml(str?: string): string {
    return str ? str
        .replace(/&/g, "&amp;")
        .replace(/</g, "&lt;")
        .replace(/>/g, "&gt;")
        .replace(/"/g, "&quot;")
        .replace(/'/g, "&#039;") : ''
  },

  /**
   * String 을 개행 문자('\r', '\n') 기준으로 나눈다.
   * @param str
   * @return 개행문자로 나누어진 string 의 array
   */
  splitNewLine(str?: string): Array<string> {
    return str ? str.split(/\r\n|\r|\n/g) : []
  },

  /**
   * 날짜를 주어진 형식의 문자열로 변환한다.
   * ````javascript
   *   formatUtil.formatDate('2020-10-04T12:34:56.000') // returns "2020.10.04"
   *   formatUtil.formatDate('2020-10-04T12:34:56.000', 'MINUTE') // returns "2020.10.04 12:34"
   *   formatUtil.formatDate(null, 'DATE') // returns ''
   *   formatUtil.formatDate(undefined, 'DATE') // returns ''
   * ````
   * @param date 날짜. string, Date, Dayjs 타입 중 하나.
   * @param pattern 'DATE', 'MINUTE', 또는 날짜 패턴 (ex: 'YYYY.MM.DD')
   * @returns pattern 형식으로 변환된 문자열. date 나 pattern 이 false 이면 빈 문자열을 리턴한다.
   */
  formatDate(date?: string | Date | Dayjs | null, pattern = 'DATE'): string {
    if (!date || !pattern) {
      return ''
    }
    if (typeof (date) === 'string') {
      date = dayjs(date)
    }
    const actualPattern = DATE_PATTERN.get(pattern) || pattern
    return dayjs(date).format(actualPattern)
  },

  /**
   * 가격/금액을 포매팅한다.
   * @param value 포매팅할 가격/금액
   * @param withUnit 화폐단위 포함 여부
   * @return 포맷팅된 가격/금액. value 가 undefined/null 이면 null.
   */
  formatPrice(value?: number | string | null, withUnit = false): string | null {
    if ((value !== undefined) && (value !== null)) {
      // TODO 소수점 이하가 있는 경우 처리
      const regexp = /\B(?=(\d{3})+(?!\d))/g
      const formatted = String(value).replace(regexp, ',')
      return withUnit ?
          `${uiComponentConfig.currencySymbolBefore}${formatted}${uiComponentConfig.currencySymbolAfter}`
          : formatted
    }
    return null
  },

  /**
   * 포인트를 포매팅한다.
   * @param value 포매팅할 포인트
   * @param withUnit 포인트단위 포함 여부
   * @return 포맷팅된 포인트. value 가 undefined/null 이면 null.
   */
  formatPoint(value?: number | string | null, withUnit = false): string | null {
    if ((value !== undefined) && (value !== null)) {
      // TODO 소수점 이하가 있는 경우 처리
      const regexp = /\B(?=(\d{3})+(?!\d))/g
      const formatted = String(value).replace(regexp, ',')
      return withUnit ?
          `${formatted}${uiComponentConfig.pointSymbolAfter}`
          : formatted
    }
    return null
  },

  /**
   * 숫자에 구분점을 넣는다.
   * @param value
   * @return
   */
  formatNumber(value?: number | string | null): string | null {
    if (value || typeof (value) === 'number') {
      const regexp = /\B(?=(\d{3})+(?!\d))/g
      return String(value).replace(regexp, ',')
    }
    return null
  },

  /**
   * rate 를 받아 percent 를 출력한다.
   * * 0.1 -> 10%
   * * 0.55 -> 55%
   * * 0.551 -> 55.1%
   * @param value 비율 값
   * @param withUnit true 이면 % 를 붙이고, false 이면 붙이지 않는다.
   * @param decimalPlace 소숫점 n번째에서 반올림한다.
   */
  formatPercent(value?: number | string | null, withUnit = true, decimalPlace = 2): string | null {
    return (value !== undefined) && (value !== null) && !isNaN(Number(value)) ?
        `${round(Number(value) * 100, decimalPlace)}${withUnit ? '%' : ''}` : null
  },

  /**
   * 신용카드번호를 4자리 단위로 나누어 구분자를 넣는다.
   * @param value
   * @param separator 구분자
   */
  formatCreditCardNo(value: string, separator = '-'): string | undefined {
    return value.match(/.{1,4}/g)?.join(separator)
  },

  /**
   *  `HTML`태그가 포함된 문자열에서 text만 남긴다.
   *  @param textIncludedHTMLElement HTML 태그를 포함하는 문자열
   *  @return HTML 태그가 제거된 문자열
   */
  toPlainText(textIncludedHTMLElement?: string): string {
    const tmp = document.createElement("DIV")
    tmp.innerHTML = textIncludedHTMLElement || ''
    return tmp.textContent || tmp.innerText
  }
}

export default formatUtil